package Servlets;

import DAO.IngredientDAO;
import Exceptions.IngredientDAOException;
import POJO.Ingredient;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@WebServlet("/ingredients/*")
public class IngredientServlet extends HttpServlet {
    private IngredientDAO ingredientDAO;
    private ObjectMapper objectMapper;

    @Override
    public void init() {
        ingredientDAO = new IngredientDAO();
        objectMapper = new ObjectMapper();
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException, ServletException {
        String pathInfo = req.getPathInfo();
        if (pathInfo == null || pathInfo.equals("/")) {
            // Get all ingredients
            try {
                List<Ingredient> ingredients = IngredientDAO.getAllIngredients();
                resp.setContentType("application/json");
                resp.getWriter().write(objectMapper.writeValueAsString(ingredients));
            } catch (Exception e) {
                resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                resp.getWriter().write("Error fetching ingredients: " + e.getMessage());
            }
        } else {
            String[] pathParts = pathInfo.split("/");
            if (pathParts.length == 2) {
                // Get ingredient by id
                try {
                    int id = Integer.parseInt(pathParts[1]);
                    Ingredient ingredient = IngredientDAO.getIngredient(id);
                    if (ingredient != null) {
                        resp.setContentType("application/json");
                        resp.getWriter().write(objectMapper.writeValueAsString(ingredient));
                    } else {
                        resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
                        resp.getWriter().write("Ingredient not found with id: " + id);
                    }
                } catch (NumberFormatException e) {
                    resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                    resp.getWriter().write("Invalid ingredient id: " + pathParts[1]);
                } catch (Exception e) {
                    resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                    resp.getWriter().write("Error fetching ingredient: " + e.getMessage());
                }
            } else if (pathParts.length == 3 && pathParts[2].equals("name")) {
                // Get ingredient name by id
                try {
                    int id = Integer.parseInt(pathParts[1]);
                    Ingredient ingredient = IngredientDAO.getIngredient(id);
                    if (ingredient != null) {
                        resp.setContentType("application/json");
                        resp.getWriter().write(objectMapper.writeValueAsString(ingredient.getName()));
                    } else {
                        resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
                        resp.getWriter().write("Ingredient not found with id: " + id);
                    }
                } catch (NumberFormatException e) {
                    resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                    resp.getWriter().write("Invalid ingredient id: " + pathParts[1]);
                } catch (Exception e) {
                    resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                    resp.getWriter().write("Error fetching ingredient name: " + e.getMessage());
                }
            } else {
                resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                resp.getWriter().write("Invalid endpoint");
            }
        }
    }


    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        // Parse the request body as a JSON object
        ObjectMapper objectMapper = new ObjectMapper();
        Ingredient ingredient;
        try {
            ingredient = objectMapper.readValue(req.getInputStream(), Ingredient.class);
        } catch (JsonProcessingException e) {
            // If the request body cannot be parsed as a JSON object, send a bad request response
            resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            resp.getWriter().write("Error parsing request body");
            return;
        }

        // Check if the ingredient already exists in the database
        Ingredient existingIngredient = null;
        try {
            existingIngredient = IngredientDAO.getIngredient(ingredient.getId());
        } catch (IngredientDAOException e) {
            throw new RuntimeException(e);
        }

        if (existingIngredient == null) {
            // If the ingredient doesn't exist, add it to the database
            try {
                IngredientDAO.insertIngredient(ingredient);
            } catch (IngredientDAOException e) {
                // If there was an error inserting the ingredient, send an internal server error response
                resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                resp.getWriter().write("Error inserting ingredient: " + e.getMessage());
                return;
            }
        } else {
            // If the ingredient already exists, update it in the database
            existingIngredient.setName(ingredient.getName());
            existingIngredient.setPrice(ingredient.getPrice());
            try {
                IngredientDAO.updateIngredient(existingIngredient);
            } catch (IngredientDAOException e) {
                // If there was an error updating the ingredient, send an internal server error response
                resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                resp.getWriter().write("Error updating ingredient: " + e.getMessage());
                return;
            }
        }

        // Send a response to the client
        resp.setStatus(HttpServletResponse.SC_CREATED);
        resp.getWriter().write("Ingredient added or updated successfully");
    }

    @Override
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        String pathInfo = req.getPathInfo();
        if (pathInfo != null && !pathInfo.equals("/")) {
            String[] pathParts = pathInfo.split("/");
            if (pathParts.length == 2) {
                // Delete ingredient by id
                try {
                    int id = Integer.parseInt(pathParts[1]);
                    ingredientDAO.deleteIngredient(id); // Create this method
                    resp.setStatus(HttpServletResponse.SC_OK);
                } catch (NumberFormatException e) {
                    resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                    resp.getWriter().write("Invalid ingredient id: " + pathParts[1]);
                } catch (Exception e) {
                    resp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                    resp.getWriter().write("Error deleting ingredient: " + e.getMessage());
                }
            } else {
                resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                resp.getWriter().write("Invalid endpoint");
            }
        } else {
            resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            resp.getWriter().write("Invalid endpoint");
        }
    }
}

/*

 */